%
% autocorr2d.m, 10 Mar 2009 15:14 (+0100)
%
% Author: D. Van De Ville
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function A0=autocorr2d(H,crit)

len1=size(H,1)/2;
len2=size(H,2)/2;

% stop criterion
%crit=1e-4;
improvement=1.0;

% initial "guess"
A0=ones(len1,len2);

% calculation loop

while improvement>crit,

  % sinc interpolation
  if 1,
  Af=fftshift(ifftn(A0));
  Af(1,:)=Af(1,:)/2;
  Af(:,1)=Af(:,1)/2;
  Af(len1+1,1:len2)=Af(1,len2:-1:1);
  Af(1:len1,len2+1)=Af(len1:-1:1,1);
  Ai=zeros(2*len1,2*len2);
  Ai(len1-len1/2+1:len1+len1/2+1,len2-len2/2+1:len2+len2/2+1)=Af;
  Ai=fftn(ifftshift(Ai));
  else
  Ai=interp2(A0,1,'nearest'); Ai(:,end+1)=Ai(:,end); Ai(end+1,:)=Ai(end,:);
  end;

  % recursion
  A1=Ai(1:len1,1:len2).*H(1:len1,1:len2)+Ai(len1+1:2*len1,1:len2).*H(len1+1:2*len1,1:len2)+Ai(1:len1,len2+1:2*len2).*H(1:len1,len2+1:2*len2)+Ai(len1+1:2*len1,len2+1:2*len2).*H(len1+1:2*len1,len2+1:2*len2);

  improvement=mean(abs(A1(:)-A0(:)));

  A0=A1;

end;

