%
% est_test.m, 04 Mar 2009 15:10 (+0100)
%
% Author: Pouya D. Tafti
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


clear;
close all;
randn('state',0);

N = 512;  % image size (If you change this you should also edit wcoefmeansq.m.)

d=2; % dim
J=8; % decomposition depth
selectsubbands=2:J; % subbands used in analysis

Nitns = 500;
epsi  = 1e-9;

I=input('number of realizations? ');
H=input('Hurst parameter?        ');

g = H/2+d/4; % gamma

H0   = zeros(I,1);
Hest = zeros(I,1);

for i=1:I,

    disp(sprintf('\n== realization %d',i));

    fBm = mfBm(d,g,N);
    %fBm = fBm(1:end/2,1:end/2);

    H0(i)   = hurstest2D(fBm,J,selectsubbands);

    Hest(i) = hurstMLest2D(fBm,J,selectsubbands,H0(i),Nitns,epsi);

end;

H0_mean   = mean(H0);
H0_std    = std(H0);

Hest_mean = mean(Hest);
Hest_std  = std(Hest);

disp(sprintf('H0:   mean = % 1.4f, std = % 2.3f',H0_mean  ,H0_std));
disp(sprintf('Hest: mean = % 1.4f, std = % 2.3f',Hest_mean,Hest_std));
