%
% hurstMLest2D.m, 04 Mar 2009 15:10 (+0100)
%
% Author: Pouya D. Tafti
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function [Hest,Cest] = hurstMLest2D(im, J, selectsubbands, H0, Nitns, epsi)

FIGURE = false;

d=2;
detD = 2;

[svar Nj] = wcoefmeansq(im,J);

svar = svar(selectsubbands);
Nj   = Nj(selectsubbands);
seng = svar.*Nj; % we don't want the mean, but the energy!

N    = sum(Nj(:));

Hest = H0;
PL   = detD.^(-selectsubbands*(2*Hest + d)/d);
Cest = (1/N) * sum(seng.*PL);

Ct = zeros(1,Nitns);
Ht = zeros(1,Nitns);

% Newton's method
itn = 1;
while true,

    disp(['itn ' num2str(itn)]);

    PL = detD.^(-selectsubbands./d.*(2*Hest + d));
    fH  = sum(selectsubbands.*(seng.*PL - Nj*Cest));
    fpH = -(2*log(detD)/d)*sum(selectsubbands.^2.*seng.*PL);

    update = -fH/fpH;
    
    Hest = Hest + update;
    Cest = (1/N) * sum(seng.*PL);

    if FIGURE,
        Ct(itn) = Cest;
        Ht(itn) = Hest;
    end;

    itn = itn+1;

    if abs(update) < epsi,
        disp('epsi');
        break;
    end;
    if itn > Nitns,
        disp('itns');
        break;
    end;

end;

if FIGURE,
    figure;
    subplot(2,1,1);
    plot(1:Nitns,Ct,'k', 'LineWidth', 1);
    ylabel('Cest');
    subplot(2,1,2);
    plot(1:Nitns,Ht,'k', 'LineWidth', 1);
    ylabel('Hest');
end;
