%
% hurstest2D.m, 04 Mar 2009 15:14 (+0100)
%
% Author: Pouya D. Tafti
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function Hest = hursest2D(im, J, selectsubbands)

FIGURE=false;

d=2;

svar = wcoefmeansq(im,J);  % correction factor for background pixels.

lsvar = log2(svar)/log2(sqrt(2));
lsvar = lsvar(:,selectsubbands);
coef = polyfit(selectsubbands,lsvar,1);
Hest = (coef(1)-d)/2;

if FIGURE,
    h = figure;
    hold on;
    plot(selectsubbands,(selectsubbands*coef(1)+coef(2)),'k', 'LineWidth', 1);
    plot(selectsubbands,lsvar,'k+', 'LineWidth', 1);
    hold off;
    grid on;
    box on;

    axis([selectsubbands(1) selectsubbands(end) -15 15]);
    set(gca, 'FontSize', 18);
    xlabel('level [$n$]', 'Interpreter', 'latex', 'FontSize', 18);
    ylabel('$\log_{\sqrt 2}\langle w_n^2[{\mathbf k}]\rangle$', 'Interpreter', 'latex', 'FontSize', 18);
%   title(sprintf( '$H_\\mathrm{est}} = %0.2f$', ...
%   Hest),'Interpreter', 'latex', 'FontSize',18);

    N = size(im,1);
    saveas(h,sprintf('fig/N%dHe%03.0f_est.eps',N,round(Hest*100)));

    figure;
    imagesc(im);
    colormap gray;
    axis square;

    plim = im - min(im(:));  plim = plim / max(plim(:));
    imwrite(plim(1:end/2,1:end/2)*64,gray,sprintf('fig/N%dHe%03.0f_gray.png',N,round(Hest*100)),'BitDepth',8);
end;
