%
% ilapkern.m, 04 Mar 2009 15:10 (+0100)
%
% Author: Pouya D. Tafti
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function krn = ilapkern(d,gamma,N,METHOD);
% d == 2, METHOD == 0 : lowpass
% d == 2, METHOD == 1 : autocorr

ITERATIONS = 10;
CRIT = 1e-4;

filename = sprintf('kern/M%0dd%0dN%0dg%03d.mat',METHOD,d,N,round(gamma*1000));
if exist(filename)
    disp('ilapkern: loading previously computed kernel...');
    load(filename,'krn');
    return;
end
disp(['ilapkern: ' filename]);
disp('ilapkern: kernel not previously computed.  please be patient...   ');

krn = zeros([repmat(N,1,d)]);

step=2*pi/N;

switch d
case 1,
    disp('not currently supported');
    return;
case 2,
    warning off MATLAB:divideByZero;
    if(METHOD == 1)
        % autocorrelation function
        [xo,yo]=meshgrid(0:step/2:2*pi-step/2);

        % refinement filter for [2 0; 0 2]
        loc_initial=(sin(xo/2).^2+sin(yo/2).^2).^(gamma);
        loc_subsampled=(sin(xo).^2+sin(yo).^2).^(gamma);
        H  = 2^(-2*gamma)*(loc_subsampled./loc_initial);
        H(find(isnan(H))) = 1;

        % reduce localization operator
        loc_initial=loc_initial(1:2:end,1:2:end);

        % autocorrelation function
        ac0 = real(autocorr2d(H.^2,CRIT));
    
        krn=ac0./(loc_initial.^2);
 
        krn(1,1) = 0;
        krn=sqrt(krn);
        warning on MATLAB:divideByZero;
    else
        w=[0;0];
        NH = floor(N/2)+1;
        ITER=0;
        for i=0:N-1
        for j=0:N-1
            w = 2*pi*[i;j]/N + 2*pi*ITER; %- 2*pi*(floor(N/2))*[1;1;1]/N;
            if i < NH
                if j < NH
                    krn(i+1,j+1) = norm(w).^(-2*gamma);
                else
                    krn(i+1,j+1)=krn(i+1,N-j+1);
                end
            else
                if j < NH
                    krn(i+1,j+1) = krn(N-i+1,j+1);
                else
                    krn(i+1,j+1) = krn(N-i+1,N-j+1);
                end
            end
        end
        end

        %   for i=1:d
        %   for j=1:d
        %   krn(:,:,i,j)=ifftshift(krn(:,:,i,j));
        %   end
        %   end
        krn(1,1,1) = 0;
    end
case 3,
    w=[0;0;0];
    for ITER=-ITERATIONS:(ITERATIONS-1)
    for i=0:N-1
    for j=0:N-1
    for k=0:N-1
       w = 2*pi*[i;j;k]/N + 2*pi*ITER; %- 2*pi*(floor(N/2))*[1;1;1]/N;
       krn(i+1,j+1,k+1) = krn(i+1,j+1,k+1) +norm(w).^(-2*gamma);
    end
    end
    end
        disp(sprintf('\b\b\b\b\b%3.0f%',50*(ITER+1)/ITERATIONS+50));
    end

%   for i=1:d
%   for j=1:d
%   krn(:,:,i,j)=ifftshift(krn(:,:,i,j));
%   end
%   end
    krn(1,1,1) = 0;
end

disp('ilapkern: saving...');
save(filename,'krn');
disp('ilapkern: done');
