%
% mfBm.m, 04 Mar 2009 15:10 (+0100)
%
% Author: Pouya D. Tafti
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function [y]=mfBm(d,gamma,N)

METHOD=0;

if (d ~= 1) && (d ~= 2) && (d ~= 3)
    disp('currently supported values for d are 1, 2, and 3');
    return;
end

% y is NxNx...xN
% the first d inices correspond to position.  the last one indexes the vector
% components.
wgn = randn(repmat(N,1,d));

WGN=fft2(wgn);

% kern is NxNx...xN
%kern = zeros([repmat(N,1,d),d,d]);
disp('mfBm: computing kernel...');
kern = ilapkern(d,gamma,N,METHOD);

disp('mfBm: applying operator...');
Y=zeros(size(WGN));

switch d
case 1,
    Y(2:end) = kern(2:end).*WGN(2:end);
    Y(1) = -real(sum(Y(2:end)));
case 2,
    Y(2:end,2:end) = kern(2:end,2:end).*WGN(2:end,2:end);
    Y(1,1) = -real(sum(reshape(squeeze(Y(2:end,2:end)),(N-1)^d,1)));
case 3,
    Y(2:end,2:end,2:end) = kern(2:end,2:end,2:end).*WGN(2:end,2:end,2:end);
    Y(1,1,1) = -real(sum(reshape(squeeze(Y(2:end,2:end,2:end)),(N-1)^d,1)));
end

y=real(ifftn(Y));

disp('mfBm: done');
